from django.contrib import admin

from textplusstuff.admin import TextPlusStuffRegisteredModelAdmin

from .models import ContentGroup, Content


class ContentInline(admin.StackedInline):
    model = Content
    extra = 0
    ordering = ['order']


class ContentGroupAdmin(TextPlusStuffRegisteredModelAdmin):
    inlines = [ContentInline]


admin.site.register(ContentGroup, ContentGroupAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import textplusstuff.fields
import versatileimagefield.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Content',
            fields=[
                (
                    'id',
                    models.AutoField(
                        verbose_name='ID',
                        serialize=False,
                        auto_created=True,
                        primary_key=True
                    )
                ),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_modified', models.DateTimeField(auto_now=True)),
                (
                    'order',
                    models.PositiveIntegerField(
                        default=1,
                        verbose_name='Order'
                    )
                ),
                (
                    'title',
                    models.CharField(
                        max_length=120,
                        verbose_name='Title'
                    )
                ),
                (
                    'menu_image',
                    versatileimagefield.fields.VersatileImageField(
                        help_text='The menu image for this content.',
                        upload_to=b'content-group/',
                        verbose_name='Menu Image'
                    )
                ),
                (
                    'alternate_menu_image',
                    versatileimagefield.fields.VersatileImageField(
                        help_text="An optional alternate menu image used for "
                                  "'hover' and 'open' states.",
                        upload_to=b'content-group/alternate_image',
                        verbose_name='Alternate Menu Image',
                        blank=True
                    )
                ),
                (
                    'menu_image_ppoi',
                    versatileimagefield.fields.PPOIField(
                        default='0.5x0.5',
                        max_length=20,
                        editable=False
                    )
                ),
                (
                    'alternate_menu_image_ppoi',
                    versatileimagefield.fields.PPOIField(
                        default='0.5x0.5',
                        max_length=20,
                        editable=False
                    )
                ),
                (
                    'content',
                    textplusstuff.fields.TextPlusStuffField(
                        help_text='The content of this section.',
                        null=True,
                        verbose_name='Content',
                        blank=True
                    )
                ),
            ],
            options={
                'ordering': ['order'],
                'verbose_name': 'Content',
                'verbose_name_plural': 'Content',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ContentGroup',
            fields=[
                (
                    'id',
                    models.AutoField(
                        verbose_name='ID',
                        serialize=False,
                        auto_created=True,
                        primary_key=True
                    )
                ),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_modified', models.DateTimeField(auto_now=True)),
                (
                    'name',
                    models.CharField(
                        help_text=(
                            b'The name of this content group. For internal '
                            b'use only.'
                        ),
                        max_length=120,
                        verbose_name='Name'
                    )
                ),
            ],
            options={
                'verbose_name': 'Content Group',
                'verbose_name_plural': 'Content Groups',
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='content',
            name='group',
            field=models.ForeignKey(
                related_name='content_set',
                verbose_name='Content Group',
                to='stacks_contentgroup.ContentGroup'
            ),
            preserve_default=True,
        ),
    ]



from django.apps import AppConfig


class StacksContentGroupConfig(AppConfig):
    """Django app config for stacks-contentgroup."""

    name = 'stacks_contentgroup'
    verbose_name = "[Stacks] Content Group"



from __future__ import unicode_literals
import json

from django.contrib.auth.models import User
from django.test import Client, TestCase

from stacks_contentgroup.models import ContentGroup


class StacksContentGroupTestCase(TestCase):
    """The test suite for stacks-contentgroup."""

    fixtures = ['stackscontentgroup.json']
    maxDiff = None

    def setUp(self):
        """Set up the test suite."""
        password = '12345'
        user = User.objects.create_user(
            username='test_user',
            email='user@test.com',
            password=password
        )
        user.is_active = True
        user.is_staff = True
        user.is_superuser = True
        user.save()
        user_client = Client()
        user_login = user_client.login(
            username='test_user',
            password=password
        )
        self.assertTrue(user_login)
        self.user = user
        self.user_client = user_client
        self.contentgroup = ContentGroup.objects.get()

    def test_instance(self):
        """Test the test StacksContentGroup instance."""
        self.assertEqual(
            self.contentgroup.pk,
            1
        )
        self.assertEqual(
            self.contentgroup.__str__(),
            'Sesame Street Characters'
        )
        self.assertEqual(
            self.contentgroup.content_set.all()[0].__str__(),
            'Sesame Street Characters - #1. Big Bird'
        )

    def test_list_serialization(self):
        """Test the StacksImageList textplusstuff serializer."""
        response = self.client.get(
            '/textplusstuff/stacks_contentgroup/'
            'contentgroup/detail/1/'
        )
        self.assertEqual(
            response.status_code,
            200
        )
        self.assertEqual(
            json.loads(response.content)['context'],
            {
                "name": "Sesame Street Characters",
                "display_title": "Sesame Street Characters",
                "extra_context": {},
                "content": [
                    {
                        "id": "sesame-street-characters-1-1",
                        "title": "Big Bird",
                        "menu_image": {
                            "square_crop": (
                                "/media/__sized__/content-group/big-bird-crop-"
                                "c0-47__0-2-490x490.jpg"
                            )
                        },
                        "alternate_menu_image": {},
                        "content": {
                            "as_plaintext": (
                                "Big Bird is a character on the children's "
                                "television show Sesame Street. Officially "
                                "performed by Caroll Spinney since 1969, he "
                                "is an eight-foot two-inch (249 cm) tall "
                                "bright primrose-yellow bird.\n"
                            ),
                            "as_html": (
                                "<p>Big Bird is a character on the children's "
                                "television show Sesame Street. Officially "
                                "performed by Caroll Spinney since 1969, he "
                                "is an eight-foot two-inch (249 cm) tall "
                                "bright primrose-yellow bird.</p>\n"
                            ),
                            "raw_text": (
                                "Big Bird is a character on the children's "
                                "television show Sesame Street. Officially "
                                "performed by Caroll Spinney since 1969, he "
                                "is an eight-foot two-inch (249 cm) tall "
                                "bright primrose-yellow bird."
                            ),
                            "as_html_no_tokens": (
                                "<p>Big Bird is a character on the children's "
                                "television show Sesame Street. Officially "
                                "performed by Caroll Spinney since 1969, he "
                                "is an eight-foot two-inch (249 cm) tall "
                                "bright primrose-yellow bird.</p>\n"
                            ),
                            "as_markdown": (
                                "Big Bird is a character on the children's "
                                "television show Sesame Street. Officially "
                                "performed by Caroll Spinney since 1969, he "
                                "is an eight-foot two-inch (249 cm) tall "
                                "bright primrose-yellow bird."
                            )
                        }
                    },
                    {
                        "id": "sesame-street-characters-1-2",
                        "title": "Cookie Monster",
                        "menu_image": {
                            "square_crop": (
                                "/media/__sized__/content-group/cookie-monster"
                                "-crop-c0-47__0-23-490x490.jpg"
                            )
                        },
                        "alternate_menu_image": {},
                        "content": {
                            "as_plaintext": (
                                "Cookie Monster is a Muppet on the long "
                                "running children's television show Sesame "
                                "Street. He is best known for his voracious "
                                "appetite and his famous eating phrases: \"Me "
                                "want cookie!\", \"Me eat cookie!\", and \"Om "
                                "nom nom nom\" (said through a mouth full of "
                                "food).\n"
                            ),
                            "as_html": (
                                "<p>Cookie Monster is a Muppet on the long "
                                "running children's television show Sesame "
                                "Street. He is best known for his voracious "
                                "appetite and his famous eating phrases: \"Me "
                                "want cookie!\", \"Me eat cookie!\", and \"Om "
                                "nom nom nom\" (said through a mouth full of "
                                "food).</p>\n"
                            ),
                            "raw_text": (
                                "Cookie Monster is a Muppet on the long "
                                "running children's television show Sesame "
                                "Street. He is best known for his voracious "
                                "appetite and his famous eating phrases: \"Me "
                                "want cookie!\", \"Me eat cookie!\", and \"Om "
                                "nom nom nom\" (said through a mouth full of "
                                "food)."
                            ),
                            "as_html_no_tokens": (
                                "<p>Cookie Monster is a Muppet on the long "
                                "running children's television show Sesame "
                                "Street. He is best known for his voracious "
                                "appetite and his famous eating phrases: \"Me "
                                "want cookie!\", \"Me eat cookie!\", and \"Om "
                                "nom nom nom\" (said through a mouth full of "
                                "food).</p>\n"
                            ),
                            "as_markdown": (
                                "Cookie Monster is a Muppet on the long "
                                "running children's television show Sesame "
                                "Street. He is best known for his voracious "
                                "appetite and his famous eating phrases: \"Me "
                                "want cookie!\", \"Me eat cookie!\", and \"Om "
                                "nom nom nom\" (said through a mouth full of "
                                "food)."
                            )
                        }
                    },
                    {
                        "id": "sesame-street-characters-1-3",
                        "title": "Oscar The Grouch",
                        "menu_image": {
                            "square_crop": (
                                "/media/__sized__/content-group/oscar-the-"
                                "grouch-crop-c0-53__0-16-490x490.png"
                            )
                        },
                        "alternate_menu_image": {},
                        "content": {
                            "as_plaintext": (
                                "Oscar the Grouch is a Muppet character on "
                                "the television program Sesame Street. He has "
                                "a green body (during the first season he was "
                                "orange), has no visible nose, and lives "
                                "in a trash can.\n"
                            ),
                            "as_html": (
                                "<p>Oscar the Grouch is a Muppet character on "
                                "the television program Sesame Street. He has "
                                "a green body (during the first season he was "
                                "orange), has no visible nose, and lives "
                                "in a trash can.</p>\n"
                            ),
                            "raw_text": (
                                "Oscar the Grouch is a Muppet character on "
                                "the television program Sesame Street. He has "
                                "a green body (during the first season he was "
                                "orange), has no visible nose, and lives "
                                "in a trash can."
                            ),
                            "as_html_no_tokens": (
                                "<p>Oscar the Grouch is a Muppet character on "
                                "the television program Sesame Street. He has "
                                "a green body (during the first season he was "
                                "orange), has no visible nose, and lives "
                                "in a trash can.</p>\n"
                            ),
                            "as_markdown": (
                                "Oscar the Grouch is a Muppet character on "
                                "the television program Sesame Street. He has "
                                "a green body (during the first season he was "
                                "orange), has no visible nose, and lives "
                                "in a trash can."
                            )
                        }
                    }
                ]
            }
        )



from __future__ import unicode_literals

import os

PROJECT_DIR = os.path.abspath(os.path.dirname(__file__))
MEDIA_ROOT = os.path.join(PROJECT_DIR, 'media')
SECRET_KEY = 'fake-key'
INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django.contrib.sessions",
    "django.contrib.staticfiles",
    "rest_framework",
    "versatileimagefield",
    "textplusstuff",
    "stacks_contentgroup",
    "tests",
]


MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
    }
}

MEDIA_URL = '/media/'
STATIC_URL = '/static/'

ROOT_URLCONF = 'tests.urls'
DEBUG = True
USE_TZ = True

TEXTPLUSSTUFF_STUFFGROUPS = {
    'stacks': {
        'name': 'Stacks Modules',
        'description': "Aggregates all Stacks Stuff."
    },
}

VERSATILEIMAGEFIELD_RENDITION_KEY_SETS = {
    'stacks_contentgroup': [
        ('square_crop', 'crop__490x490')
    ]
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('stacks_contentgroup', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='contentgroup',
            name='display_title',
            field=models.CharField(default='Foo', help_text=b'The displayed-to-the-user title of this Content Group.', max_length=120, verbose_name='Title'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='content',
            name='title',
            field=models.CharField(help_text=b'The displayed-to-the-user title of this content.', max_length=120, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='contentgroup',
            name='name',
            field=models.CharField(help_text=b'The internal name/signifier of this content.', max_length=120, verbose_name='Name'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def populate_display_title(apps, schema_editor):
    """
    Populates display_title for ContentGroup
    """
    ContentGroup = apps.get_model("stacks_contentgroup", "ContentGroup")
    for cg in ContentGroup.objects.all():
        cg.display_title = cg.name
        cg.save()


class Migration(migrations.Migration):

    dependencies = [
        ('stacks_contentgroup', '0002_auto_20150528_1526'),
    ]

    operations = [
        migrations.RunPython(
            populate_display_title
        )
    ]



from django.conf import settings
from django.utils.text import slugify

from rest_framework import serializers
from textplusstuff.serializers import (
    ExtraContextSerializerMixIn,
    TextPlusStuffFieldSerializer
)
from versatileimagefield.serializers import VersatileImageFieldSerializer

from .models import Content, ContentGroup

image_sets = getattr(
    settings,
    'VERSATILEIMAGEFIELD_RENDITION_KEY_SETS',
    {}
).get(
    'stacks_contentgroup',
    [
        ('full_size', 'url'),
        ('3up_thumb', 'crop__700x394'),
        ('2up_thumb', 'crop__800x450'),
        ('full_width', 'crop__1600x901'),
    ]
)


class ContentSerializer(serializers.ModelSerializer):
    id = serializers.SerializerMethodField()
    menu_image = VersatileImageFieldSerializer(
        sizes=image_sets
    )
    alternate_menu_image = VersatileImageFieldSerializer(
        sizes=image_sets
    )
    content = TextPlusStuffFieldSerializer()

    class Meta:
        model = Content
        fields = (
            'id',
            'title',
            'menu_image',
            'alternate_menu_image',
            'content'
        )

    def get_id(self, obj):
        return "{}-{}-{}".format(
            slugify(obj.group.display_title),
            obj.group.pk,
            obj.pk
        )


class ContentGroupSerializer(ExtraContextSerializerMixIn,
                             serializers.ModelSerializer):
    content = ContentSerializer(source='content_set', many=True)

    class Meta:
        model = ContentGroup
        fields = (
            'display_title',
            'name',
            'content'
        )



from textplusstuff import registry

from .models import ContentGroup
from .serializers import ContentGroupSerializer


class ContentGroupStuff(registry.ModelStuff):
    queryset = ContentGroup.objects.prefetch_related('content_set')
    verbose_name = 'Content Group'
    verbose_name_plural = 'Content Groups'
    description = 'Display a Content Group'
    serializer_class = ContentGroupSerializer
    renditions = [
        registry.Rendition(
            short_name='3up',
            verbose_name='Content Group 3-Up',
            description='Displays the Content Group in stacks of three.',
            path_to_template='stacks_contentgroup/contentgroup/'
                             'contentgroup-3up.html',
            rendition_type='block',
        ),
        registry.Rendition(
            short_name='2up',
            verbose_name='Content Group 2-Up',
            description='Displays the Content Group in stacks of two.',
            path_to_template='stacks_contentgroup/contentgroup/'
                             'contentgroup-2up.html',
            rendition_type='block'
        )
    ]
    list_display = ('id', 'name')

registry.stuff_registry.add_modelstuff(
    ContentGroup,
    ContentGroupStuff,
    groups=['stacks', ]
)



from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from textplusstuff.fields import TextPlusStuffField
from versatileimagefield.fields import VersatileImageField, PPOIField


class ContentBase(models.Model):
    """A base model for shared attributes/logic across stacks_page models."""

    date_created = models.DateTimeField(
        auto_now_add=True
    )
    date_modified = models.DateTimeField(
        auto_now=True
    )

    class Meta:
        abstract = True


@python_2_unicode_compatible
class ContentGroup(ContentBase):
    """A group of Content instances."""
    name = models.CharField(
        _('Name'),
        max_length=120,
        help_text="The internal name/signifier of this content."
    )
    display_title = models.CharField(
        _('Title'),
        max_length=120,
        help_text='The displayed-to-the-user title of this Content Group.',
        blank=True
    )

    class Meta:
        verbose_name = _('Content Group')
        verbose_name_plural = _('Content Groups')

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Content(ContentBase):
    """Represents a block of content."""

    order = models.PositiveIntegerField(
        _('Order'),
        default=1
    )
    group = models.ForeignKey(
        ContentGroup,
        verbose_name=_('Content Group'),
        related_name='content_set'
    )
    title = models.CharField(
        _('Title'),
        max_length=120,
        help_text='The displayed-to-the-user title of this content.'
    )
    menu_image = VersatileImageField(
        _('Menu Image'),
        upload_to="content-group/",
        ppoi_field="menu_image_ppoi",
        help_text=_("The menu image for this content.")
    )
    alternate_menu_image = VersatileImageField(
        _('Alternate Menu Image'),
        upload_to="content-group/alternate_image",
        ppoi_field="alternate_menu_image_ppoi",
        help_text=_(
            "An optional alternate menu image used for 'hover' "
            "and 'open' states."
        ),
        blank=True
    )
    menu_image_ppoi = PPOIField()
    alternate_menu_image_ppoi = PPOIField()
    content = TextPlusStuffField(
        _('Content'),
        blank=True,
        null=True,
        help_text=_('The content of this section.')
    )

    class Meta:
        verbose_name = _('Content')
        verbose_name_plural = _('Content')
        ordering = ['order']

    def __str__(self):
        return "{} - #{}. {}".format(
            self.group.name,
            self.order,
            self.title
        )



from __future__ import unicode_literals

from django.conf.urls import patterns, include, url

from django.contrib import admin

from textplusstuff.registry import stuff_registry, findstuff
admin.autodiscover()
findstuff()

urlpatterns = patterns(
    '',
    url(r'^textplusstuff/', include(stuff_registry.urls)),
    url(r'^admin/', include(admin.site.urls))
)



# -*- coding: utf-8 -*-
from distutils.core import setup
from setuptools import find_packages

setup(
    name='stacks-contentgroup',
    packages=find_packages(),
    version='0.1.2',
    author=u'Jonathan Ellenberger',
    author_email='jonathan_ellenberger@wgbh.org',
    url='http://stacks.wgbhdigital.org/',
    license='MIT License, see LICENSE',
    description=(
        "A Stacks apps for creating groups of menu-ed content."
    ),
    long_description=open('README.md').read(),
    zip_safe=False,
    install_requires=[
        'django-versatileimagefield>=1.0.2',
        'django-textplusstuff>=0.4',
        'stacks-page>=0.1.1'
    ],
    package_data={
        'stacks_contentgroup': [
            'static/sass/*.scss',
            'static/js/*.js',
            'templates/stacks_contentgroup/contentgroup/*.html'
        ]
    },
    classifiers=[
        'Framework :: Django',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Development Status :: 3 - Alpha'
    ]
)
